@@ -732,9 +732,14 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 		mstruct = vargs[0];
 		if(!mstruct.isVector()) {
 			if(mstruct.representsScalar()) return 1;
+			CALCULATOR->beginTemporaryStopMessages();
 			mstruct.eval(eo);
-			mstruct = vargs[0];
-			if(!mstruct.isVector()) return 1;
+			if(!mstruct.isVector()) {
+				CALCULATOR->endTemporaryStopMessages();
+				mstruct = vargs[0];
+				return 1;
+			}
+			CALCULATOR->endTemporaryStopMessages(true);
 		}
 		mstruct.setType(STRUCT_FUNCTION);
 		mstruct.setFunction(this);
@@ -1223,9 +1228,13 @@ int GenerateVectorFunction::calculate(MathStructure &mstruct, const MathStructur
 	if(!b_step) {
 		CALCULATOR->beginTemporaryStopMessages();
 		msteps.eval(eo);
-		if(vargs[5].number().isNegative()) {
-			b_step = !msteps.isInteger() || msteps.number().isNegative() || msteps.number().isOne();
+		if(msteps.isNumber() && !msteps.number().isInteger() && msteps.number().isInterval() && !msteps.number().isNonInteger() && msteps.number().precision(true) > PRECISION + 10) {
+			Number nr;
+			if(msteps.number().getCentralInteger(nr)) {
+				msteps.set(nr, true);
+			}
 		}
+		if(vargs[5].number().isNegative()) b_step = !msteps.isInteger() || msteps.number().isNegative() || msteps.number().isOne();
 		CALCULATOR->endTemporaryStopMessages(!b_step);
 	}
 	if(b_step) {
@@ -1233,7 +1242,7 @@ int GenerateVectorFunction::calculate(MathStructure &mstruct, const MathStructur
 	} else {
 		bool overflow = false;
 		int steps = msteps.number().intValue(&overflow);
-		if(!msteps.isNumber() || overflow || steps < 1) {
+		if(!msteps.isInteger() || overflow || steps < 1) {
 			CALCULATOR->error(true, _("The number of requested elements in generate vector function must be a positive integer."), NULL);
 			return 0;
 		}
@@ -1307,7 +1316,7 @@ LoadFunction::LoadFunction() : MathFunction("load", 1, 3) {
 	setArgumentDefinition(2, arg);
 	setDefaultValue(2, "1");
 	setArgumentDefinition(3, new TextArgument());
-	setDefaultValue(3, "\",\"");
+	setDefaultValue(3, ",");
 }
 int LoadFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	string delim = vargs[2].symbol();
@@ -1324,7 +1333,7 @@ ExportFunction::ExportFunction() : MathFunction("export", 2, 3) {
 	setArgumentDefinition(1, new MatrixArgument());
 	setArgumentDefinition(2, new FileArgument());
 	setArgumentDefinition(3, new TextArgument());
-	setDefaultValue(3, "\",\"");
+	setDefaultValue(3, ",");
 }
 int ExportFunction::calculate(MathStructure&, const MathStructure &vargs, const EvaluationOptions&) {
 	string delim = vargs[2].symbol();
