@@ -379,7 +379,7 @@ DataSet::DataSet(string s_category, string s_name, string s_default_file, string
 	b_loaded = false;
 	setArgumentDefinition(1, new DataObjectArgument(this, _("Object")));
 	setArgumentDefinition(2, new DataPropertyArgument(this, _("Property")));
-	setDefaultValue(2, _("info"));
+	setDefaultValue(2, _c("info", "Data set argument"));
 	setChanged(false);
 }
 DataSet::DataSet(const DataSet *o) {
@@ -539,13 +539,17 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 	}
 	gsub("-", "_", locale);
 #else
-	char *clocale = setlocale(LC_MESSAGES, NULL);
+	char *clocale = getenv("LANGUAGE");
+	if(!clocale || strlen(clocale) == 0) clocale = setlocale(LC_MESSAGES, NULL);
+	if(!clocale || strlen(clocale) == 0) clocale = getenv("LANG");
 	if(clocale) locale = clocale;
 #endif
 	if(CALCULATOR->getIgnoreLocale() || locale == "POSIX" || locale == "C") {
 		locale = "";
 	} else {
-		size_t i = locale.find('.');
+		size_t i = locale.find(':');
+		if(i != string::npos) locale = locale.substr(0, i);
+		i = locale.find('.');
 		if(i != string::npos) locale = locale.substr(0, i);
 	}
 
@@ -702,9 +706,9 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 								value = xmlNodeListGetString(doc, child->xmlChildrenNode, 1);
 								lang = xmlNodeGetLang(child);
 								ils = -1;
-								for(int i3 = lang_status_p.size() - 1; i3 > 0; i3--) {
-									if(lang_status_p[i3] == properties[i3]) {
-										ils = i3;
+								for(int i3 = lang_status_p.size(); i3 > 0; i3--) {
+									if(lang_status_p[i3 - 1] == properties[i3 - 1]) {
+										ils = i3 - 1;
 										break;
 									}
 								}
