@@ -35,7 +35,32 @@
 #define INSERT_REF(o, i)	{MathStructure *m_append = o; v_order.insert(v_order.begin() + i, v_subs.size()); v_subs.push_back(m_append); m_append->ref(); if(!b_approx && m_append->isApproximate()) b_approx = true; if(m_append->precision() > 0 && (i_precision < 1 || m_append->precision() < i_precision)) i_precision = m_append->precision();}
 #define CLEAR			v_order.clear(); for(size_t i = 0; i < v_subs.size(); i++) {v_subs[i]->unref();} v_subs.clear();
 //#define REDUCE(v_size)		for(size_t v_index = v_size; v_index < v_order.size(); v_index++) {v_subs[v_order[v_index]]->unref(); v_subs.erase(v_subs.begin() + v_order[v_index]);} v_order.resize(v_size);
-#define REDUCE(v_size)          {std::vector<size_t> v_tmp; v_tmp.resize(SIZE, 0); for(size_t v_index = v_size; v_index < v_order.size(); v_index++) {v_subs[v_order[v_index]]->unref(); v_subs[v_order[v_index]] = NULL; v_tmp[v_order[v_index]] = 1;} v_order.resize(v_size); for(std::vector<MathStructure*>::iterator v_it = v_subs.begin(); v_it != v_subs.end();) {if(*v_it == NULL) v_it = v_subs.erase(v_it); else ++v_it;} size_t i_change = 0; for(size_t v_index = 0; v_index < v_tmp.size(); v_index++) {if(v_tmp[v_index] == 1) i_change++; v_tmp[v_index] = i_change;} for(size_t v_index = 0; v_index < v_order.size(); v_index++) v_order[v_index] -= v_tmp[v_index];}
+#define REDUCE(v_size)          {\
+	for(size_t v_index = v_size; v_index < v_order.size(); v_index++) {\
+		v_subs[v_order[v_index]]->unref();\
+	}\
+	v_order.resize(v_size, 0);\
+	bool b_reorder_subs = false;\
+	for(size_t v_index = 0; v_index < v_order.size(); v_index++) {\
+		if(v_order[v_index] != v_index) {\
+			b_reorder_subs = true;\
+			break;\
+		}\
+	}\
+	if(b_reorder_subs) {\
+		std::vector<MathStructure*> v_subs_new;\
+		v_subs_new.resize(v_order.size(), NULL);\
+		for(size_t v_index = 0; v_index < v_order.size(); v_index++) {\
+			v_subs_new[v_index] = v_subs[v_order[v_index]];\
+		}\
+		v_subs = v_subs_new;\
+		for(size_t v_index = 0; v_index < v_order.size(); v_index++) {\
+			v_order[v_index] = v_index;\
+		}\
+	} else {\
+		v_subs.resize(v_order.size(), NULL);\
+	}\
+}
 #define CHILD(v_index)		(*v_subs[v_order[v_index]])
 #define SIZE			v_order.size()
 #define LAST			(*v_subs[v_order[v_order.size() - 1]])
@@ -53,6 +78,37 @@
 
 #define FUNCTION_PROTECTED(evalops, id) (evalops.protected_function != NULL && evalops.protected_function == CALCULATOR->getFunctionById(id))
 
+#ifndef CLOCK_MONOTONIC
+#	define PREPARE_TIMECHECK_VAR struct timeval tv_end;
+#	define PREPARE_TIMECHECK_TIME(ms) \
+					gettimeofday(&tv_end, NULL); \
+					tv_end.tv_usec += ((ms) % 1000) * 1000; \
+					tv_end.tv_sec += ((ms) / 1000); \
+					if(tv_end.tv_usec >= 1000000L) { \
+						tv_end.tv_sec++; \
+						tv_end.tv_usec -= 1000000L; \
+					}
+#	define DO_TIMECHECK \
+					struct timeval tv; \
+					gettimeofday(&tv, NULL); \
+					if(tv.tv_sec > tv_end.tv_sec || (tv.tv_sec == tv_end.tv_sec && tv.tv_usec >= tv_end.tv_usec))
+#else
+#	define PREPARE_TIMECHECK_VAR struct timespec tv_end;
+#	define PREPARE_TIMECHECK_TIME(ms) \
+					clock_gettime(CLOCK_MONOTONIC, &tv_end); \
+					tv_end.tv_nsec += ((ms) % 1000) * 1000000L; \
+					tv_end.tv_sec += ((ms) / 1000); \
+					if(tv_end.tv_nsec >= 1000000000L) { \
+						tv_end.tv_sec++; \
+						tv_end.tv_nsec -= 1000000000L; \
+					}
+#	define DO_TIMECHECK \
+					struct timespec tv; \
+					clock_gettime(CLOCK_MONOTONIC, &tv); \
+					if(tv.tv_sec > tv_end.tv_sec || (tv.tv_sec == tv_end.tv_sec && tv.tv_nsec >= tv_end.tv_nsec))
+#endif
+#	define PREPARE_TIMECHECK(ms) PREPARE_TIMECHECK_VAR PREPARE_TIMECHECK_TIME(ms)
+
 void printRecursive(const MathStructure &mstruct);
 
 std::string format_and_print(const MathStructure &mstruct);
@@ -103,6 +159,8 @@ int test_comparisons(const MathStructure &msave, MathStructure &mthis, const Mat
 bool replace_function(MathStructure &m, MathFunction *f1, MathFunction *f2, const EvaluationOptions &eo);
 bool replace_intervals_f(MathStructure &mstruct);
 bool replace_f_interval(MathStructure &mstruct, const EvaluationOptions &eo);
+bool calculate_rand(MathStructure &mstruct, const EvaluationOptions &eo);
+bool contains_rand(const MathStructure &mstruct, bool check_variables = false);
 bool fix_intervals(MathStructure &mstruct, const EvaluationOptions &eo, bool *failed = NULL, long int min_precision = 2, bool function_middle = false);
 bool set_uncertainty(MathStructure &mstruct, MathStructure &munc, const EvaluationOptions &eo = default_evaluation_options, bool do_eval = false);
 bool create_interval(MathStructure &mstruct, const MathStructure &m1, const MathStructure &m2);
