@@ -176,9 +176,9 @@ bool Unit::isUsedByOtherUnits() const {
 }
 string Unit::print(const PrintOptions &po, bool format, int tagtype, bool input, bool plural) const {
 	if(input) {
-		preferredInputName(po.abbreviate_names, po.use_unicode_signs, plural, po.use_reference_names || (po.preserve_format && isCurrency()), po.can_display_unicode_string_function, po.can_display_unicode_string_arg).formattedName(TYPE_UNIT, !po.use_reference_names && tagtype != TAG_TYPE_TERMINAL, format && tagtype == TAG_TYPE_HTML, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
+		preferredInputName(po.abbreviate_names, po.use_unicode_signs, plural, po.use_reference_names || (po.preserve_format && isCurrency()), po.can_display_unicode_string_function, po.can_display_unicode_string_arg).formattedName(TYPE_UNIT, !po.use_reference_names, format && tagtype == TAG_TYPE_HTML, format && tagtype == TAG_TYPE_TERMINAL && po.use_unicode_signs ? 1 : 0, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
 	}
-	return preferredDisplayName(po.abbreviate_names, po.use_unicode_signs, plural, po.use_reference_names || (po.preserve_format && isCurrency()), po.can_display_unicode_string_function, po.can_display_unicode_string_arg).formattedName(TYPE_UNIT, !po.use_reference_names && tagtype != TAG_TYPE_TERMINAL, format && tagtype == TAG_TYPE_HTML, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
+	return preferredDisplayName(po.abbreviate_names, po.use_unicode_signs, plural, po.use_reference_names || (po.preserve_format && isCurrency()), po.can_display_unicode_string_function, po.can_display_unicode_string_arg).formattedName(TYPE_UNIT, !po.use_reference_names, format && tagtype == TAG_TYPE_HTML, format && tagtype == TAG_TYPE_TERMINAL && po.use_unicode_signs ? 1 : 0, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
 }
 string Unit::print(bool plural_, bool short_, bool use_unicode, bool (*can_display_unicode_string_function) (const char*, void*), void *can_display_unicode_string_arg) const {
 	return preferredName(short_, use_unicode, plural_, false, can_display_unicode_string_function, can_display_unicode_string_arg).name;
@@ -305,8 +305,6 @@ bool Unit::convert(Unit *u, MathStructure &mvalue, MathStructure &mexp) const {
 	if(u == this) {
 		return true;
 	} else if(u->baseUnit() == baseUnit()) {
-		u->convertToBaseUnit(mvalue, mexp);
-		convertFromBaseUnit(mvalue, mexp);
 		if(isCurrency() && u->isCurrency()) {
 			int i = 0;
 			if(u->subtype() == SUBTYPE_ALIAS_UNIT && u->isBuiltin()) {
@@ -341,7 +339,17 @@ bool Unit::convert(Unit *u, MathStructure &mvalue, MathStructure &mexp) const {
 					i = i | 0b0100;
 				}
 			}
+			if(isBuiltin() && u->isBuiltin() && u->subtype() == SUBTYPE_ALIAS_UNIT && subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseUnit() == ((AliasUnit*) this)->firstBaseUnit()) {
+				((AliasUnit*) u)->convertToBaseCurrencyAlt(mvalue, mexp);
+				((AliasUnit*) this)->convertFromBaseCurrencyAlt(mvalue, mexp);
+			} else {
+				u->convertToBaseUnit(mvalue, mexp);
+				convertFromBaseUnit(mvalue, mexp);
+			}
 			CALCULATOR->setExchangeRatesUsed(i);
+		} else {
+			u->convertToBaseUnit(mvalue, mexp);
+			convertFromBaseUnit(mvalue, mexp);
 		}
 		return true;
 	}
@@ -408,6 +416,7 @@ string AliasUnit::expression() const {
 	return svalue;
 }
 string AliasUnit::inverseExpression() const {
+	if(!sinverse.empty() && isBuiltin() && o_unit == CALCULATOR->getUnitById(UNIT_ID_EURO) && i_exp == 1) return "";
 	return sinverse;
 }
 string AliasUnit::uncertainty(bool *is_relative) const {
@@ -484,10 +493,10 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 	if(i_exp != 1) mexp /= i_exp;
 	ParseOptions po;
 	if(isApproximate() && suncertainty.empty() && precision() == -1) {
-		if(sinverse.find(DOT) != string::npos || svalue.find(DOT) != string::npos) po.read_precision = READ_PRECISION_WHEN_DECIMALS;
+		if(svalue.find(DOT) != string::npos || inverseExpression().find(DOT) != string::npos) po.read_precision = READ_PRECISION_WHEN_DECIMALS;
 		else po.read_precision = ALWAYS_READ_PRECISION;
 	}
-	if(sinverse.empty()) {
+	if(inverseExpression().empty()) {
 		if(svalue.find("\\x") != string::npos) {
 			string stmp = svalue;
 			string stmp2 = LEFT_PARENTHESIS ID_WRAP_LEFT;
@@ -628,6 +637,15 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 	}
 	return mvalue;
 }
+MathStructure &AliasUnit::convertFromBaseCurrencyAlt(MathStructure &mvalue, MathStructure &mexp) const {
+	if(sinverse.empty() || !inverseExpression().empty()) return convertFromBaseUnit(mvalue, mexp);
+	ParseOptions po;
+	MathStructure *mstruct = new MathStructure();
+	CALCULATOR->parse(mstruct, sinverse, po);
+	if(isApproximate() && !mstruct->isApproximate()) mstruct->setApproximate(true, true);
+	mvalue.divide_nocopy(mstruct, true);
+	return mvalue;
+}
 MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
 	ParseOptions po;
 	if(isApproximate() && suncertainty.empty() && precision() == -1) {
@@ -724,6 +742,20 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 	if(i_exp != 1) mexp.multiply(i_exp);
 	return mvalue;
 }
+MathStructure &AliasUnit::convertToBaseCurrencyAlt(MathStructure &mvalue, MathStructure &mexp) const {
+	if(sinverse.empty() || !inverseExpression().empty()) return convertToBaseUnit(mvalue, mexp);
+	ParseOptions po;
+	MathStructure *mstruct = new MathStructure();
+	CALCULATOR->parse(mstruct, sinverse, po);
+	if(isApproximate() && !mstruct->isApproximate()) mstruct->setApproximate(true, true);
+	if(mvalue.isOne()) {
+		mvalue.set_nocopy(*mstruct);
+		mstruct->unref();
+	} else {
+		mvalue.multiply_nocopy(mstruct, true);
+	}
+	return mvalue;
+}
 void AliasUnit::setExponent(int exp) {
 	i_exp = exp;
 	setChanged(true);
@@ -885,7 +917,7 @@ string AliasUnit_Composite::print(const PrintOptions &po, bool format, int tagty
 		ename = &o_unit->preferredDisplayName(po.abbreviate_names, po.use_unicode_signs, plural, po.use_reference_names || (po.preserve_format && o_unit->isCurrency()), po.can_display_unicode_string_function, po.can_display_unicode_string_arg);
 		if(prefixv) str = prefixv->preferredDisplayName(ename->abbreviation, po.use_unicode_signs, plural, po.use_reference_names, po.can_display_unicode_string_function, po.can_display_unicode_string_arg).name;
 	}
-	str += ename->formattedName(TYPE_UNIT, !po.use_reference_names && tagtype != TAG_TYPE_TERMINAL, format && tagtype == TAG_TYPE_HTML, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
+	str += ename->formattedName(TYPE_UNIT, !po.use_reference_names, format && tagtype == TAG_TYPE_HTML, format && tagtype == TAG_TYPE_TERMINAL && po.use_unicode_signs ? 1 : 0, !po.use_reference_names && !po.preserve_format, po.hide_underscore_spaces);
 	return str;
 }
 string AliasUnit_Composite::print(bool plural_, bool short_, bool use_unicode, bool (*can_display_unicode_string_function) (const char*, void*), void *can_display_unicode_string_arg) const {
@@ -1282,6 +1314,7 @@ bool replace_variables(MathStructure &m) {
 			if(m.variable()->referenceName() == "bohr_radius") u = CALCULATOR->getActiveUnit("bohr_unit");
 			else if(m.variable()->referenceName() == "elementary_charge") u = CALCULATOR->getActiveUnit("e_unit");
 			else if(m.variable()->referenceName() == "electron_mass") u = CALCULATOR->getActiveUnit("electron_unit");
+			else if(m.variable()->referenceName() == "compton_wavelength_2pi") u = CALCULATOR->getActiveUnit("l_N");
 		}
 		if(u) {
 			m.set(u, true);
@@ -1321,9 +1354,17 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 	}
 	remove_times_one(mstruct);
 	fix_division(mstruct, eo);
-	bool b_eval = !is_unit_multiexp(mstruct);
+	bool b_eval = !is_unit_multiexp(mstruct) && mstruct.containsType(STRUCT_UNIT, false, true, true) != 0;
 	while(true) {
-		if(b_eval) mstruct.eval(eo);
+		if(b_eval) {
+			if(CALCULATOR->isControlled()) {
+				mstruct.eval(eo);
+			} else {
+				CALCULATOR->startControl(100);
+				mstruct.eval(eo);
+				CALCULATOR->stopControl();
+			}
+		}
 		if(mstruct.isUnit()) {
 			add(mstruct.unit(), 1, mstruct.prefix());
 		} else if(mstruct.isPower() && mstruct[0].isUnit() && mstruct[1].isInteger()) {
@@ -1361,10 +1402,10 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 	}
 	if(conversion_variant && had_errors) {
 		CALCULATOR->endTemporaryStopMessages();
-		CALCULATOR->error(true, _("Error(s) in unitexpression."), NULL);
+		CALCULATOR->error(true, _("Error(s) in unit expression."), NULL);
 	} else {
 		if(CALCULATOR->endTemporaryStopMessages() > 0) had_errors = true;
-		if(had_errors) CALCULATOR->error(false, _("Error(s) in unitexpression."), NULL);
+		if(had_errors) CALCULATOR->error(false, _("Error(s) in unit expression."), NULL);
 	}
 	setChanged(true);
 }
